#!/usr/bin/env python

import os
from optparse import OptionParser, OptionGroup
import __builtin__

from crebs.config       import get_defaults_config
from crebs.debug        import report


def introduction(blurb):

    fitted_blurb = ''
    blurb_array = []
    blurb_line = '    '

    blurb = blurb.split()
    for word in blurb:
        if blurb_line == '    ':
            blurb_line += word
        elif 1 + len(blurb_line + word) <= 72:
            blurb_line += ' ' + word
        else:
            blurb_array.append(blurb_line)
            blurb_line = '    ' + word
    blurb_array.append(blurb_line)

    for blurb_line in blurb_array:
        fitted_blurb += blurb_line + "\n"

    return fitted_blurb


def parse_inv_args():
    """Parses any invocation arguments given to crebs, and sets up default data."""

    # Fetch default data
    d = get_defaults_config()

    # Create options parser
    usage = _("Usage") + ": %prog [" + _("options") + "] [" + _("list of files") + "]"
    description = _("Create Background Slidehow takes a list of images and uses them to create a wallpaper slideshow for the GNOME desktop background, saved as an XML file. Acceptable image formats are JPEG, PNG, GIF, and SVG; other slideshows can also be listed with the images.")
    parser = OptionParser(usage=usage, version=APP_LONG_NAME, description=description)

    # Create options groups

    ### Timing options
    og_time = OptionGroup(parser, _("Timing options"),
                        _("These options affect the timings of images in the slideshow"))
    og_time.add_option("-d", _("--def-display"),
                        dest = "default_duration",
                        type = "float",
                        default = d['default_duration'],
                        help = _("set the default display duration in seconds"),
                        metavar = _("SECONDS"))
    og_time.add_option("-t", _("--def-transition"),
                        dest = "default_transition",
                        type = "float",
                        default = d['default_transition'],
                        help = _("set the default transition time in seconds"),
                        metavar = _("SECONDS"))
    og_time.add_option("-e", _("--display-times"),
                        dest = "specific_durations",
                        default = "",
                        help = _("specify the display times for each image with a comma-separated LIST of times in seconds; empty items and hyphens are interpreted as 'use default'"),
                        metavar = _("LIST"))
    og_time.add_option("-u", _("--transition-times"),
                        dest = "specific_transitions",
                        default = "",
                        help = _("specify the transition times for each image with a comma-separated LIST of times in seconds; empty items and hyphens are interpreted as 'use default'"),
                        metavar = _("LIST"))

    ### Slideshow options
    og_slideshow = OptionGroup(parser, _("Slideshow options"),
                        _("These options affect the slideshow itself"))
    og_slideshow.add_option("-n", _("--title"),
                        dest = "name",
                        default = "",
                        help = _("specify the NAME for the slideshow; this is a title, not a filename"),
                        metavar = _("NAME"))
    og_slideshow.add_option("-r", _("--only-create"),
                        dest = "register_wallpaper",
                        action = "store_false",
                        default = d['register_wallpaper'],
                        help = _("do not register the slideshow as an available wallpaper; implicitly set by -g without one of -n, -m, or -o"))
    og_slideshow.add_option("-R", _("--register-wallpaper"),
                        dest = "register_wallpaper",
                        action = "store_true",
                        default = d['register_wallpaper'],
                        help = _("register the slideshow as an available wallpaper; overrides -r"))
    og_slideshow.add_option("-w", _("--keep-wallpaper"),
                        dest = "set_wallpaper",
                        action = "store_false",
                        default = d['set_wallpaper'],
                        help = _("do not change the wallpaper after creating the slideshow; implicitly set by -r"))
    og_slideshow.add_option("-W", _("--set-wallpaper"),
                        dest = "set_wallpaper",
                        action = "store_true",
                        default = d['set_wallpaper'],
                        help = _("change the wallpaper after creating the slideshow; overrides -w"))
    og_slideshow.add_option("-y", _("--style"),
                        dest = "wallpaper_style",
                        type = "string",
                        default = d['wallpaper_style'],
                        help = _("specify the style used to display the wallpaper; one of 'wallpaper', 'zoom', 'centered', 'scaled', 'stretched', and 'spanned'"),
                        metavar = _("STYLE"))

    ### File options
    og_file = OptionGroup(parser, _("File options"),
                        _("These options affect the slideshow file name and location"))
    og_file.add_option("-l", _("--location"),
                        dest = "directory",
                        type = "string",
                        default = "", # Default directory left blank for later options test
                        help = _("specify the DIRECTORY in which to save slideshows; relative paths are assumed to be in your home directory"),
                        metavar = _("DIRECTORY"))
    og_file.add_option("-m", _("--filename"),
                        dest = "filename",
                        type = "string",
                        default = "",
                        help = _("save slideshow to FILE within the slideshow directory; if the GUI is shown, this option will cause it to close once the slideshow has been created"),
                        metavar = _("FILE"))
    og_file.add_option("-o", _("--output"),
                        dest = "output",
                        type = "string",
                        default = "",
                        help = _("specify DIRECTORY and FILENAME for the slideshow in a single path; relative paths are assumed to be in your home directory; if the GUI is shown, this option will cause it to close once the slideshow has been created"),
                        metavar = _("DIRECTORY/FILENAME"))
    og_file.add_option("-f", _("--force-overwrite"),
                        dest = "force_overwrite",
                        action = "store_true",
                        default = d['force_overwrite'],
                        help = _("overwrite existing slideshows with the same filename without asking"))
    og_file.add_option("-F", _("--warn-overwrite"),
                        dest = "force_overwrite",
                        action = "store_false",
                        default = d['force_overwrite'],
                        help = _("give warning of existing slideshows before overwriting them; overrides -f"))
    og_file.add_option("-s", _("--standard-output"),
                        dest = "standard_output",
                        action = "store_true",
                        default = d['standard_output'],
                        help = _("write the slideshow file to standard output; use with -c to pipe background XML data to another program"))
    og_file.add_option("-S", _("--no-standard-output"),
                        dest = "standard_output",
                        action = "store_false",
                        default = d['standard_output'],
                        help = _("do not write the slideshow file to standard output; overrides -s"))
    og_file.add_option("-c", _("--no-files"),
                        dest = "file_creation",
                        action = "store_false",
                        default = d['file_creation'],
                        help = _("no files will be created; use with -s to pipe background XML data to another program"))
    og_file.add_option("-C", _("--create-files"),
                        dest = "file_creation",
                        action = "store_true",
                        default = d['file_creation'],
                        help = _("create slideshow files; overrides -c"))

    ### GUI options
    og_gui = OptionGroup(parser, _("GUI options"),
                        _("These options affect the graphical user interface (GUI)"))
    og_gui.add_option("-g", _("--no-gui"),
                        dest = "gui",
                        action = "store_false",
                        default = True,
                        help = _("do not show the GUI; a name or filename is required in order to set the wallpaper"))
    og_gui.add_option("-q", _("--no-notifications"),
                        dest = "notify",
                        action = "store_false",
                        default = d['notify'],
                        help = _("do not send a notification when wallpaper is set"))
    og_gui.add_option("-Q", _("--send-notifications"),
                        dest = "notify",
                        action = "store_true",
                        default = d['notify'],
                        help = _("send a notification when wallpaper is set; overrides -q"))
    og_gui.add_option(_("--experimental-features"),
                        dest = "experimental_features",
                        action = "store_true",
                        default = d['experimental_features'],
                        help = _("activate the experimental features; these may not function perfectly"))

    # Add options to parser
    parser.add_option("-v", _("--verbose"),
                        dest = "verbosity",
                        action = "count",
                        help = _("be more verbose; use more than once for extra verbosity; messages are written to the standard error stream"))

    parser.add_option_group(og_time)
    parser.add_option_group(og_slideshow)
    parser.add_option_group(og_file)
    parser.add_option_group(og_gui)

    # Parse options and arguments
    (options, files) = parser.parse_args()

    __builtin__.VERBOSITY = options.verbosity

    # Check options for errors
    if options.output and (options.directory or options.filename):
        parser.error(_("option -o may not be used with -l or -m"))

    if not options.file_creation and (options.directory or options.filename or options.output):
        parser.error(_("option -c may not be used with -l, -m, or -o"))

    if options.wallpaper_style not in ('wallpaper', 'zoom', 'centered', 'scaled', 'stretched', 'spanned'):
        parser.error(_("option -y must be one of 'wallpaper', 'zoom', 'centered', 'scaled', 'stretched', or 'spanned'"))

    # Start the application
    intro_text =  APP_LONG_NAME + "\n"

    intro_text += _("Copyright (C) 2010 Marc Stewart") + "\n"
    if _("Copyright (C) 2010 Marc Stewart") != "Copyright (C) 2010 Marc Stewart":
        intro_text += "Copyright (C) 2010 Marc Stewart\n"

    intro_text += introduction(_("This program comes with ABSOLUTELY NO WARRANTY."))
    if _("This program comes with ABSOLUTELY NO WARRANTY.") != "This program comes with ABSOLUTELY NO WARRANTY.":
        intro_text += introduction("This program comes with ABSOLUTELY NO WARRANTY.")

    intro_text += introduction(_("This is free software, and you are welcome to redistribute it under certain conditions; see COPYING text file for details."))
    if _("This is free software, and you are welcome to redistribute it under certain conditions; see COPYING text file for details.") != "This is free software, and you are welcome to redistribute it under certain conditions; see COPYING text file for details.":
        intro_text += introduction("This is free software, and you are welcome to redistribute it under certain conditions; see COPYING text file for details.")

    if not(options.gui or options.standard_output):
        # Display if CLI AND no standard output
        print intro_text
    else:
        report(1, intro_text)

    ### Non-GUI requires a name
    if not options.gui:
        if not (options.name or options.filename or options.output):
            options.file_creation = False
            report(4, "Options: file_creation implicitly unset by -g without -m|n|o")
            options.register_wallpaper = False
            report(4, "Options: register_wallpaper implicitly unset by -g without -m|n|o")
        ### file creation needed to set wallpaper
        elif not options.file_creation:
            options.register_wallpaper = False
            report(4, "Options: register_wallpaper implicitly unset by -gc")
        ### notification requires wallpaper to be set
        if (not options.register_wallpaper) or (not options.set_wallpaper):
            options.set_wallpaper = False
            options.notify = False
            report(4, "Options: notify implicitly unset")

    ### Default directory
    if not (options.output or options.directory):
        options.directory = d['slideshows_directory']
        report(4, "Options: default directory will be used")

    ### Directory trailing slash
    if options.directory:
        if options.directory[-1] != '/':
            options.directory += '/'
            report(4, "Options: trailing slash added to directory")

        ### Absolute directory path
        if options.directory[0] != '/':
            options.directory = os.path.join(os.getenv("HOME"), options.directory)
            report(4, "Options: relative directory assumed to be in user's home")

    ### Absolute output path
    if options.output:
        if options.output[0] != '/':
            options.output = os.path.join(os.getenv("HOME"), options.output)
            report(4, "Options: relative output path assumed to be in user's home")

    ### Default duration
    try:
        dur = float(options.default_duration)
    except ValueError:
        dur = default_duration
        report(4, "Options: default duration not a number")

    ### Default transition
    try:
        trans = float(options.default_transition)
    except ValueError:
        trans = default_transition
        report(4, "Options: default transition not a number")
        
    ### Specific durations and transitions
    durations = []
    transitions = []
    options.specific_durations = options.specific_durations.split(',')
    options.specific_transitions = options.specific_transitions.split(',')
    for i in range(len(files)):
        try:
            dur = options.specific_durations[i]
        except IndexError:
            dur = -1
            report(4, "Options: specific_durations[{0}] not specified".format(i))
        else:
            try:
                dur = float(dur)
            except ValueError:
                dur = -1
                report(4, "Options: specific_durations[{0}] not a number".format(i))
        finally:
            durations.append(dur)

        try:
            trans = options.specific_transitions[i]
        except IndexError:
            trans = -1
            report(4, "Options: specific_transitions[{0}] not specified".format(i))
        else:
            try:
                trans = float(trans)
            except ValueError:
                trans = -1
                report(4, "Options: specific_transitions[{0}] not a number".format(i))
        finally:
            transitions.append(trans)

    options.specific_durations = durations
    options.specific_transitions = transitions

    # Add extra remaining defaults to options
    options.images_directory = d['images_directory']

    return (files, options)
