#!/usr/bin/env python

from os.path            import exists, basename, join

from crebs.filesystem   import make_needed_dirs, safe_filename
from crebs.imagelists   import create_image_list
from crebs.slideshows   import create_slideshow_xml, write_slideshow
from crebs.wallpapers   import register_wallpaper, set_wallpaper
from crebs.notify       import send_notification
from crebs.debug        import report, report_options


def check_file_options(options):
    """Checks the file options and fills in any details."""

    report(3, "Checking the file options...")

    if options.output:
        options.filename = basename(options.output)

    else:
        if options.filename:
            options.output = join(options.directory, options.filename)

        elif options.name:
            options.filename = safe_filename(options.name)
            options.output = join(options.directory, options.filename)

        else:
            options.file_creation = False
            report(3, "Not enough information to write slideshow file.", 1)

    report(3, "Success.", 0.5)

    return options


def get_slideshow_name(options):
    """Returns the slideshow's name."""

    if options.name:
        s_name = options.name
    else:
        s_name = options.filename

    return s_name


def cli_warn_file_overwrite(s_filename):
    """Warns that a file with the same name already exists."""

    # FUTURE: prompt user

    report(2, _("Error: File already exists: {0}").format(s_filename), 0, True)

    return False


def cli(files, options):
    """Makes a slideshow without using the GUI."""

    # Set up data
    image_list = create_image_list(files, options)

    # FUTURE: Ask user for any extra info

    # Check options
    options = check_file_options(options)
    if len(image_list) < 2 and options.register_wallpaper:
        options.register_wallpaper = False
        report(4, 'Options: set wallpaper implicitly unset by fewer than 2 images')

    # Debugging information
    report_options(5, options, files, image_list)

    # Create slideshow
    s_xml = create_slideshow_xml(image_list, options.name)

    # Print to standard output
    if options.standard_output or options.verbosity >= 5:
        print s_xml

    # Check for overwrite
    if options.file_creation:

        if exists(options.output) and not options.force_overwrite:
            ok_to_write = cli_warn_file_overwrite(options.output)
        else:
            ok_to_write = True

        # Write slideshow
        if ok_to_write:
            make_needed_dirs(options.output)

            slideshow_written = write_slideshow(options.output, s_xml)

            # Register wallpaper
            if slideshow_written and options.register_wallpaper:
                s_name = get_slideshow_name(options)

                register_wallpaper(s_name, options.output)
                # set_wallpaper not dependent on success

                # Set wallpaper
                if options.set_wallpaper:
                    wallpaper_set = set_wallpaper(options.output)

                    # Send notification
                    if wallpaper_set and options.notify:
                        send_notification(s_name)

