#!/usr/bin/env python

from os.path import exists, expanduser
import gconf


def set_gconf_value(value_type, key_name, value):
    """Sets a specific key in GConf"""

    try:
        client = gconf.client_get_default()

        if value_type == 'float':
            success = client.set_float('/apps/crebs/' + key_name, value)
        elif value_type == 'int':
            success = client.set_int('/apps/crebs/' + key_name, value)
        elif value_type == 'string':
            success = client.set_string('/apps/crebs/' + key_name, value)
        elif value_type == 'bool':
            success = client.set_bool('/apps/crebs/' + key_name, value)
        else:
            # invalid value_type
            return False
    except:
        # error encountered
        return False
    else:
        # key-set attempted
        return success


def get_gconf_value(value_type, key_name, default_value, set_key = False):
    """Gets a specific value from GConf, creating the key if requested."""

    # Connect to GConf
    try:
        client = gconf.client_get_default()
    except:
        # unable to communicate with GConf
        return default_value
    else:
        # Get value
        try:
            if value_type == 'float':
                v = client.get_float('/apps/crebs/' + key_name)
            elif value_type == 'int':
                v = client.get_int('/apps/crebs/' + key_name)
            elif value_type == 'string':
                v = client.get_string('/apps/crebs/' + key_name)
            elif value_type == 'bool':
                v = client.get_bool('/apps/crebs/' + key_name)
            else:
                # invalid value_type requested
                return default_value
        except:
            # unable to read GConf value
            return default_value
        else:
            if v:
                # value read successfully
                return v
            else:
            # v == False, so check key has been set
                try:
                    # attempt to read the key as if it were another type
                    #   only works for unset keys
                    if value_type == 'float':
                        test = client.get_bool('/apps/crebs/' + key_name)
                    elif value_type == 'int':
                        test = client.get_bool('/apps/crebs/' + key_name)
                    elif value_type == 'string':
                        test = client.get_bool('/apps/crebs/' + key_name)
                    elif value_type == 'bool':
                        test = client.get_string('/apps/crebs/' + key_name)
                except:
                    # type mismatch error confirms key is set
                    return v
                else:
                    # Key hasn't been set, so set it if asked
                    if set_key:
                        set_gconf_value(value_type, key_name, default_value)

                    return default_value


def get_defaults_config():
    """Loads configuration from GConf, creating it if it doesn't exist yet."""

    # Define default defaults
    d_default_duration = 900.0
    d_default_transition = 3.0

    d_directories_images = expanduser('~')
    d_directories_slideshows = expanduser('~/.crebs')

    d_files_standard_output = False
    d_files_create_files = True
    d_files_force_overwrite = False

    d_wallpaper_style = 'zoom'
    d_wallpaper_register_wallpaper = True
    d_wallpaper_set_wallpaper = True

    d_notify_send_notification = True

    d_experimental_features = False

    # Look for GConf values
    default_duration = get_gconf_value('float', 'times/default_duration', d_default_duration, True)
    default_transition = get_gconf_value('float', 'times/default_transition', d_default_transition, True)

    directories_images = get_gconf_value('string', 'directories/images', d_directories_images, True)
    directories_slideshows = get_gconf_value('string', 'directories/slideshows', d_directories_slideshows, True)

    files_standard_output = get_gconf_value('bool', 'files/standard_output', d_files_standard_output, True)
    files_create_files = get_gconf_value('bool', 'files/create_files', d_files_create_files, True)
    files_force_overwrite = get_gconf_value('bool', 'files/force_overwrite', d_files_force_overwrite, True)

    wallpaper_style = get_gconf_value('string', 'wallpaper/style', d_wallpaper_style, True)
    wallpaper_register_wallpaper = get_gconf_value('bool', 'wallpaper/register_wallpaper', d_wallpaper_register_wallpaper, True)
    wallpaper_set_wallpaper = get_gconf_value('bool', 'wallpaper/set_wallpaper', d_wallpaper_set_wallpaper, True)

    notify_send_notification = get_gconf_value('bool', 'notify/send_notification', d_notify_send_notification, True)

    experimental_features = get_gconf_value('bool', 'experimental/experimental_features', d_experimental_features, True)

    # Validate values
    ### Default duration (0.1 seconds <= d <= 92 days)
    if default_duration < 0.1 or default_duration > 7948800:
        if default_duration < 0.1:
            default_duration = 0.1
        else:
            default_duration = 7948800

        set_gconf_value('float', 'times/default_duration', default_duration)

    ### Default transition (0 <= t <= default duration)
    if default_transition < 0 or default_transition > default_duration:
        if default_transition < 0:
            default_transition = 0
        else:
            default_transition = default_duration

        set_gconf_value('float', 'times/default_transition', default_transition)

    ### Images directory
    ##### Assume relative directories are in user's home
    if directories_images[0] == '/':
        set_conf = False
    else:
        set_conf = True
        if directories_images[0] != '~':
            directories_images = '~/' + directories_images

        directories_images = expanduser(directories_images)

    ##### Ensure existence
    if not exists(directories_images):
        set_conf = True
        directories_images = d_directories_images

    ##### Make correction
    if set_conf:
        set_gconf_value('string', 'directories/images', directories_images)

    ### Slideshow directory
    ##### Must not be blank
    if not directories_slideshows:
        set_conf = True
        directories_slideshows = d_directories_slideshows

    ##### Assume relative directories are in user's home
    elif directories_slideshows[0] == '/':
        set_conf = False
    else:
        set_conf = True
        if directories_slideshows[0] != '~':
            directories_slideshows = '~/' + directories_slideshows

        directories_slideshows = expanduser(directories_slideshows)

    ##### Make correction
    if set_conf:
        set_gconf_value('string', 'directories/slideshows', directories_slideshows)

    ### Display style
    if wallpaper_style not in ('wallpaper', 'zoom', 'centered', 'scaled', 'stretched', 'spanned'):
        wallpaper_style = d_wallpaper_style
        set_gconf_value('string', 'wallpaper/style', wallpaper_style)

    # Package and return
    d = { \
        'default_duration'      : default_duration, \
        'default_transition'    : default_transition, \
        'images_directory'      : directories_images, \
        'slideshows_directory'  : directories_slideshows, \
        'standard_output'       : files_standard_output, \
        'file_creation'         : files_create_files, \
        'force_overwrite'       : files_force_overwrite, \
        'wallpaper_style'       : wallpaper_style, \
        'register_wallpaper'    : wallpaper_register_wallpaper, \
        'set_wallpaper'         : wallpaper_set_wallpaper, \
        'notify'                : notify_send_notification, \
        'experimental_features' : experimental_features \
        }

    return d

