#!/usr/bin/env python

import os
from os.path import join, exists, isdir, basename, dirname
import urllib

try:
    import gtk
except:
    sys.exit(1)

from crebs.times        import get_scaled_duration
from crebs.filesystem   import safe_filename, make_needed_dirs
from crebs.imagelists   import create_gui_image_list, convert_to_gui_image_list
from crebs.thumbnails   import get_thumbnail
from crebs.slideshows   import is_slideshow_file, import_slideshow, create_slideshow_xml, write_slideshow
from crebs.wallpapers   import register_wallpaper, set_wallpaper
from crebs.notify       import send_notification
from crebs.debug        import report


# Image-list constants
IL_FILENAME = 0
IL_THUMBNAIL = 1
IL_USE_DEF_DURATION = 2
IL_DUR_SCALED = 3
IL_DUR_TIMESCALE = 4
IL_USE_DEF_TRANS = 5
IL_TRANSITION = 6


class CrebsGui:
    """Creates the GUI."""

    def __init__(self, files, options):
        """Sets up default data and GUI."""

        report(3, "Creating CreBS GUI class...")

        # Terminal conditions
        if options.output or options.filename:
            self.close_afterwards = True
        else:
            self.close_afterwards = False

        # Timings
        self.scales = (3600, 60, 1)
        self.default_duration = options.default_duration
        self.default_transition = options.default_transition
        (self.default_duration_scaled, self.default_scale) = get_scaled_duration(self.default_duration)

        # Copy options to GUI
        self.verbosity          = options.verbosity
        self.name               = options.name
        self.register_wallpaper = options.register_wallpaper
        self.set_wallpaper      = options.set_wallpaper
        self.wallpaper_style    = options.wallpaper_style
        self.directory          = options.directory
        self.filename           = options.filename
        self.output             = options.output
        self.force_overwrite    = options.force_overwrite
        self.standard_output    = options.standard_output
        self.file_creation      = options.file_creation
        self.notify             = options.notify

        # Image list: filename; icon; use def. dur.; dur./scale; scale; use def. trans.; trans
        (self.gui_image_list, self.failed_list) = create_gui_image_list(files, options)

        # Set up GUI
        b = gtk.Builder()

        b.add_from_file(join(GLADE_DIR, 'create-background-slideshow.glade'))
        b.add_from_file(join(GLADE_DIR, 'add-images.glade'))
        b.add_from_file(join(GLADE_DIR, 'clear-images.glade'))
        b.add_from_file(join(GLADE_DIR, 'file-overwrite.glade'))
        b.add_from_file(join(GLADE_DIR, 'import-error.glade'))
        b.add_from_file(join(GLADE_DIR, 'fit-error.glade'))
        b.add_from_file(join(GLADE_DIR, 'save-error.glade'))
        b.add_from_file(join(GLADE_DIR, 'set-error.glade'))

        # Connect to GUI signals
        b.connect_signals({
            'on_wndMain_destroy'    : gtk.main_quit,
            "on_drop"               : self.add_by_drop,

            "on_select"             : self.on_select_image,
            "on_unselect"           : self.on_unselect_image,
            "on_iconview_keyboard"  : self.icvImages_keyboard,

            "on_add"                : self.add_by_dialog,
            "on_copy"               : self.copy_image,
            "on_earlier"            : self.move_image_earlier,
            "on_later"              : self.move_image_later,
            "on_remove"             : self.remove_image,
            "on_clear"              : self.clear_images,

            "on_new_def_time"       : self.change_default_duration,
            "on_new_def_scale"      : self.change_default_timescale,
            "on_new_def_trans"      : self.change_default_transition,

            "on_new_img_time"       : self.change_selected_duration,
            "on_new_img_scale"      : self.change_selected_timescale,
            "on_new_img_trans"      : self.change_selected_transition,

            "on_new_length"         : self.change_slideshow_length,

            "on_toggle_length"      : self.toggle_length,
            "on_toggle_time"        : self.toggle_time,
            "on_toggle_trans"       : self.toggle_trans,

            "on_named"              : self.name_changed,
            "on_enter"              : self.try_to_save,
            "on_save"               : self.save_slideshow,

            "on_file_activated"     : self.file_activated,
            "on_file_preview"       : self.preview_image,
            "on_icvImages_resized"  : self.on_imagelist_resized,
            "on_cmb_resize"         : self.on_combobox_resize
            })

        # Fetch GUI objects
        ### Main CreBS window
        self.wndMain            = b.get_object("wndMain")
        self.icvImages          = b.get_object("icvImages")
        self.btnAdd             = b.get_object("btnAdd")
        self.btnCopy            = b.get_object("btnCopy")
        self.btnEarlier         = b.get_object("btnEarlier")
        self.btnLater           = b.get_object("btnLater")
        self.btnRemove          = b.get_object("btnRemove")
        self.btnClear           = b.get_object("btnClear")

        self.nbkTimings         = b.get_object("nbkTimings")

        self.lblDurDefault      = b.get_object("lblDisplayDefault")
        self.lblSelected        = b.get_object("lblSpecifics")
        self.lblDurSelected     = b.get_object("lblDisplaySpecific")
        self.lblTrnSelected     = b.get_object("lblTransitionSpecific")

        self.spnDurDefault      = b.get_object("spnTimeDefault")
        self.spnDurSelected     = b.get_object("spnTimeSpecific")
        self.spnTrnSelected     = b.get_object("spnTransSpecific")

        self.DurDefault         =              self.spnDurDefault.get_adjustment()
        self.TrnDefault         = b.get_object("spnTransDefault").get_adjustment()
        self.DurSelected        =             self.spnDurSelected.get_adjustment()
        self.TrnSelected        =             self.spnTrnSelected.get_adjustment()

        self.cmbDurScaleDefault = b.get_object("cmbScaleTimeDefault")
        self.cmbDurScaleSelected = b.get_object("cmbScaleTimeSpecific")
        self.lstScalesDefault   = b.get_object("lstScalesDefault")
        self.lstScalesSelected  = b.get_object("lstScalesSpecific")

        self.chkUseDefDur       = b.get_object("chkUseDefDisplay")
        self.chkUseDefTrn       = b.get_object("chkUseDefTransition")

        self.lblSlideshowLength = b.get_object("lblSlideshowLength")
        self.spnSlideshowLength = b.get_object("spnSlideshowLength")
        self.SlideshowLength    =            self.spnSlideshowLength.get_adjustment()
        self.cmbSlideshowLength = b.get_object("cmbSlideshowLength")
        self.lstScalesSlideshow = b.get_object("lstScalesSlideshow")
        self.chkFitToLength     = b.get_object("chkFitToLength")

        self.cmbSlideshowStyle  = b.get_object("cmbSlideshowStyle")
        self.lstWallpaperStyles = b.get_object("lstWallpaperStyles")

        self.lblSecondsDefault  = b.get_object("lblSecondsDefault")
        self.lblSecondsSelected = b.get_object("lblSecondsSpecific")

        self.lblName            = b.get_object("lblName")
        self.txtName            = b.get_object("txtName")
        self.Name               =          self.txtName.get_text
        self.btnSave            = b.get_object("btnSave")

        ### 'Add Images' dialog
        self.dlgAddImages       = b.get_object("dlgAddImages")

        ### 'Clear Images' dialog
        self.dlgConfirmClear    = b.get_object("dlgConfirmClear")
        self.btnKeep            = b.get_object("btnKeep")

        ### 'File Overwrite' dialog
        self.dlgOverwrite       = b.get_object("dlgConfirmOverwrite")
        self.btnDontOverwrite   = b.get_object("btnDontOverwrite")

        ### 'Import Error' dialog
        self.dlgImportError     = b.get_object("dlgImportError")

        ### 'Fit Error' dialog
        self.dlgFitError        = b.get_object("dlgFitError")

        ### 'Save Error' dialog
        self.dlgSaveError       = b.get_object("dlgSaveError")

        ### 'Set Error' dialog
        self.dlgSetError        = b.get_object("dlgSetError")
        self.btnSetErrOK        = b.get_object("btnSetErrOK")
        self.btnSetErrCancel    = b.get_object("btnSetErrCancel")

        # Set window as drag-drop destination
        self.wndMain.drag_dest_set( gtk.DEST_DEFAULT_MOTION |
                                    gtk.DEST_DEFAULT_HIGHLIGHT |
                                    gtk.DEST_DEFAULT_DROP,
                                    [ ( 'text/uri-list', 0, 80 ) ],
                                    gtk.gdk.ACTION_COPY )

        # 'Add Images' dialog
        ### File filters
        self.add_file_filters_to_add_dialog()
        ### Directory
        self.add_images_dir = options.images_directory
        ### Preview
        self.file_preview = gtk.Image()
        self.dlgAddImages.set_preview_widget(self.file_preview)
        self.dlgAddImages.set_use_preview_label(False)

        # Add images to the icon-view widget
        self.icvImages.set_model(self.gui_image_list)
        self.icvImages.set_pixbuf_column(IL_THUMBNAIL)
        self.icvImages.set_tooltip_column(IL_FILENAME)

        # Set up the 'Style' combobox
        for i in range(len(self.lstWallpaperStyles)):
            if self.lstWallpaperStyles[i][1] == self.wallpaper_style:
                self.cmbSlideshowStyle.set_active(i)
                break

        # Configure spinbuttons
        self.SlideshowLength.set_all(0, 0, 3600, 1, 10, 0)
        self.DurDefault.set_all (self.default_duration_scaled, 0.1, 3600, 1, 10, 0)
        self.DurSelected.set_all(self.default_duration_scaled, 0.1, 3600, 1, 10, 0)
        self.cmbDurScaleDefault.set_active (self.default_scale)
        self.cmbDurScaleSelected.set_active(self.default_scale)
        self.TrnDefault.set_all (self.default_transition, 0, 86400, 1, 10, 0)
        self.TrnSelected.set_all(self.default_transition, 0, 86400, 1, 10, 0)
        self.limit_duration('default')
        self.limit_duration('selected')

        # Set up constant size for comboboxes
        self.cmb_size = self.cmbDurScaleDefault.get_allocation()

        # Fill in any provided name
        if options.name:
            self.txtName.set_text(options.name)

        # Set inactive widgets and tooltips
        self.use_gui_for_name = True
        if not options.file_creation:
            self.lblName.set_sensitive(False)
            self.txtName.set_sensitive(False)
            self.use_gui_for_name = False
            self.btnSave.set_tooltip_text(_("Create this slideshow"))

            if options.standard_output:
                self.txtName.set_text(_("Standard Output only"))
            else:
                self.txtName.set_text(_("No files will be created"))

        elif not options.register_wallpaper:
            if options.filename or options.output:
                self.lblName.set_sensitive(False)
                self.txtName.set_sensitive(False)
                self.use_gui_for_name = False
                if options.output:
                    self.txtName.set_text(options.output)
                else:
                    self.txtName.set_text(options.filename)
                self.btnSave.set_tooltip_text(_("Save this slideshow and exit"))
            else:
                self.btnSave.set_tooltip_text(_("Save this slideshow"))

        elif not options.set_wallpaper:
            if options.filename or options.output:
                self.btnSave.set_tooltip_text(_("Register this slideshow and exit"))
            else:
                self.btnSave.set_tooltip_text(_("Register this slideshow"))

        # Slideshow tab
        self.toggle_length()

        self.on_unselect_image()
        self.check_if_clearable()
        self.check_if_saveable()
        self.select_page('DEFAULTS')

        report(3, "Success.", 0.5)

        # Experimental features
        if options.experimental_features:
            self.chkFitToLength.show()
        else:
            self.chkFitToLength.hide()


    def warn_import_error(self, failed_list):
        """Displays an error dialog listing the files that could not be imported."""

        fail_count = len(failed_list)
        if fail_count > 8:
            error_msg = N_("One file could not be imported.", "{0} files were unable to be imported.", fail_count).format(fail_count) + "\n"
        else:
            error_msg = N_("This file could not be imported:", "These files could not be imported:", fail_count) + "\n"
            for filename in failed_list:
                error_msg += "{0}\n".format(filename)
        error_msg += "\n" + _("Only JPEG, PNG, GIF, and SVG images, and other slideshows containing these image types, can be used for your slideshow.")

        self.dlgImportError.format_secondary_markup(error_msg)
        self.dlgImportError.run()
        self.dlgImportError.hide()
        
    def show(self):
        """Displays the GUI and starts the GTK Main Loop."""

        # Show the main window
        self.wndMain.show()

        # Warn of any errors in the command-line files list
        if self.failed_list:
            self.warn_import_error(self.failed_list)

        # Ensure comboboxes don't resize, by finding maximum width
        dds = self.default_duration_scaled

        for i in range(1, 21):
            self.DurDefault.set_value(i)

        self.DurDefault.set_value(dds)

        # Start the GTK main loop
        gtk.main()


    def add_file_filters_to_add_dialog(self):
        """Adds image and slideshow file filters to 'Add Images' dialog."""

        fil_img = gtk.FileFilter()
        fil_img.set_name(_("Images"))
        fil_img.add_mime_type("image/jpeg")
        fil_img.add_mime_type("image/png")
        fil_img.add_mime_type("image/gif")
        fil_img.add_mime_type("image/svg")
        fil_img.add_mime_type("image/svg+xml")

        fil_bg = gtk.FileFilter()
        fil_bg.set_name(_("Slideshows"))
        fil_bg.add_custom(gtk.FILE_FILTER_FILENAME, is_slideshow_file)

        fil_all = gtk.FileFilter()
        fil_all.set_name(_("All supported types"))
        fil_all.add_mime_type("image/jpeg")
        fil_all.add_mime_type("image/png")
        fil_all.add_mime_type("image/gif")
        fil_all.add_mime_type("image/svg")
        fil_all.add_mime_type("image/svg+xml")
        fil_all.add_custom(gtk.FILE_FILTER_FILENAME, is_slideshow_file)

        self.dlgAddImages.add_filter(fil_all)
        self.dlgAddImages.add_filter(fil_img)
        self.dlgAddImages.add_filter(fil_bg)


    def preview_image(self, fc):
        """Creates a preview of the selected file."""

        filename = fc.get_preview_filename()
        if filename:
            pixbuf = get_thumbnail(filename, 128)
        else:
            pixbuf = False

        if not pixbuf:
            fc.set_preview_widget_active(False)
        else:
            self.file_preview.set_from_pixbuf(pixbuf)
            fc.set_preview_widget_active(True)


    def on_imagelist_resized(self, widget, allocation):
        """Responds to changes in the imagelist size"""

        w = allocation.width
        t = 96 # thumbnail size

        n = int((w-14) / (t+20))
        r = (w-14) % (t+20)
        x = int(r / (n-1))

        self.icvImages.set_column_spacing(6 + x)


    def on_combobox_resize(self, widget, allocation):
        """Prevents comboboxes from being reduced in size."""

        if allocation.width < self.cmb_size.width:
            widget.set_size_request(self.cmb_size.width, -1)
        else:
            self.cmb_size = allocation


    def select_page(self, page):
        """Selects a page in the Timings notebook"""

        if page == 'SELECTED':
            self.nbkTimings.set_current_page(2)
        elif page == 'DEFAULTS':
            self.nbkTimings.set_current_page(1)
        elif page == 'SLIDESHOW':
            self.nbkTimings.set_current_page(0)
        else:
            report(0, 'Programmer Error: Invalid page name')


    def check_if_clearable(self):
        """Checks if there are any images to clear."""

        if len(self.gui_image_list) == 0:
            self.btnClear.set_sensitive(False)
            self.select_page('DEFAULTS')
            self.SlideshowLength.set_value(0)
            self.cmbSlideshowLength.set_active(3)
            self.wndMain.set_focus(self.btnAdd)
        else:
            self.btnClear.set_sensitive(True)


    def check_if_saveable(self):
        """Checks there are at least two images, and has a name."""

        self.check_if_fittable()

        if len(self.gui_image_list) >= 2 and self.Name() != "":
            self.btnSave.set_sensitive(True)
            return True
        else:
            self.btnSave.set_sensitive(False)
            return False


    def check_if_fittable(self):
        """Checks if there is at least one image."""

        if len(self.gui_image_list) >= 1:
            self.chkFitToLength.set_sensitive(True)
            self.change_slideshow_length()
            return True
        else:
            self.chkFitToLength.set_sensitive(False)
            self.chkFitToLength.set_active(False)
            return False


    def get_selection(self):
        """Returns the position (first=0) of the selected image."""

        if len(self.icvImages.get_selected_items()) == 0:
            return -1
        else:
            return self.icvImages.get_selected_items()[0][0]


    def is_selected_image_first(self):
        """Returns True if the selected image is first in the slideshow."""

        if self.get_selection() == 0:
            return True
        else:
            return False


    def is_selected_image_last(self):
        """Returns True if the selected image is last in the slideshow."""

        if self.get_selection() + 1 == len(self.gui_image_list):
            return True
        else:
            return False


    def activate_movement_section(self, Active):
        """Sets the sensitivity of the movement arrows."""

        self.btnRemove.set_sensitive(Active)

        if Active:
            if self.is_selected_image_first():
                self.btnEarlier.set_sensitive(False)
            else:
                self.btnEarlier.set_sensitive(True)

            if self.is_selected_image_last():
                self.btnLater.set_sensitive(False)
            else:
                self.btnLater.set_sensitive(True)
        else:
            self.btnEarlier.set_sensitive(False)
            self.btnLater.set_sensitive(False)


    def activate_selection_section(self, Active):
        """Sets the sensitivity of the 'Selected Image' section."""

        self.lblSelected.set_sensitive(Active)
        self.lblDurSelected.set_sensitive(Active)

        self.chkUseDefDur.set_sensitive(Active)

        if self.chkUseDefDur.get_active():
            self.spnDurSelected.set_sensitive(False)
            self.cmbDurScaleSelected.set_sensitive(False)
        else:
            self.spnDurSelected.set_sensitive(Active)
            self.cmbDurScaleSelected.set_sensitive(Active)

        self.lblTrnSelected.set_sensitive(Active)
        self.chkUseDefTrn.set_sensitive(Active)

        if self.chkUseDefTrn.get_active():
            self.spnTrnSelected.set_sensitive(False)
            self.lblSecondsSelected.set_sensitive(False)
        else:
            self.spnTrnSelected.set_sensitive(Active)
            self.lblSecondsSelected.set_sensitive(Active)


    def on_select_image(self, widget=False, event=False):
        """An image from the slideshow has been selected."""

        if self.get_selection() < 0 or len(self.gui_image_list) == 0:
            self.on_unselect_image()
        else:
            self.icvImages.set_cursor(self.icvImages.get_selected_items()[0])
            self.btnCopy.set_sensitive(True)
            self.activate_movement_section(True)
            self.activate_selection_section(True)
            self.select_page('SELECTED')

    	    # Update selection section with image timings/defaultness.
            sel = self.get_selection()

            if self.gui_image_list[sel][IL_USE_DEF_DURATION]:
                self.chkUseDefDur.set_active(True)
                self.DurSelected.set_value(self.DurDefault.get_value())
                self.cmbDurScaleSelected.set_active(self.cmbDurScaleDefault.get_active())
            else:
                self.chkUseDefDur.set_active(False)
                self.DurSelected.set_value(self.gui_image_list[sel][IL_DUR_SCALED])
                self.cmbDurScaleSelected.set_active(self.gui_image_list[sel][IL_DUR_TIMESCALE])

            if self.gui_image_list[sel][IL_USE_DEF_TRANS]:
                self.chkUseDefTrn.set_active(True)
                self.TrnSelected.set_value(self.TrnDefault.get_value())
            else:
                self.chkUseDefTrn.set_active(False)
                self.TrnSelected.set_value(self.gui_image_list[sel][IL_TRANSITION])


    def on_unselect_image(self):
        """Inactivates widgets when no images are selected."""

        self.btnCopy.set_sensitive(False)
        self.activate_movement_section(False)
        self.activate_selection_section(False)

        self.chkUseDefDur.set_active(True)
        self.DurSelected.set_value(self.DurDefault.get_value())
        self.cmbDurScaleSelected.set_active(self.cmbDurScaleDefault.get_active())
        self.chkUseDefTrn.set_active(True)
        self.TrnSelected.set_value(self.TrnDefault.get_value())


    def add_images_to_gui(self, filenames, indent_offset = 0):
        """Adds selected images to GUI image list."""

        image_list = []

        prev_img_count = len(self.gui_image_list)

        for filename in filenames:
            if not is_slideshow_file(filename, True, indent_offset):
                image_list.append((filename, self.default_duration, self.default_transition))

            else:
                slides = import_slideshow(filename, indent_offset)
                for slide in slides:
                    image_list.append(slide)

        (self.gui_image_list, self.failed_list) = convert_to_gui_image_list(image_list, self.default_duration, self.default_transition, self.gui_image_list)

        if self.failed_list:
            self.warn_import_error(self.failed_list)

        # Select the first of the new images
        if len(self.gui_image_list) > prev_img_count:
            self.icvImages.select_path(prev_img_count)

        self.check_if_saveable()
        self.check_if_clearable()


    def add_by_drop(self, widget, context, x, y, selection, target_type, timestamp):
        """Add an image by dropping it onto the window."""

        report(3, "File(s) dropped onto main window.")

        if target_type == 80:
            report(3, "Processing dropped files...")
            filenames=[]
            path_list = selection.data.strip('\r\n\x00').split()
            for path in path_list:
                if path.startswith('file://'):
                    path = path[7:]
                elif path.startswith('file:'):
                    path = path[5:]
                # Fix characters escaped by Nautilus, e.g. %20 -> space
                path = urllib.url2pathname(path)
                filenames.append(path)

            self.add_images_to_gui(filenames, 1)
            report(3, "Success.", 0.5)


    def file_activated(self, widget):
        """Responds to activating a file in the 'Add Images' dialog."""

        self.dlgAddImages.response(26)


    def add_by_dialog(self, widget):
        "Adds images after being chosen in the 'Add Images' dialog."""

        # Open to last-used directory (or default if first open)
        self.dlgAddImages.set_current_folder(self.add_images_dir)

        # Wait for a valid selection or cancellation
        done = False
        while done == False:
            response = self.dlgAddImages.run()

            if response != 26:
                done = True

            else:
                filenames = self.dlgAddImages.get_filenames()

                # Ignore empty selections
                if not filenames:
                    valid = False
                else:
                    valid = True

                    # Check for a directory to open instead
                    for filename in filenames:
                        if isdir(filename):
                            # Open a folder if it's the only thing selected, but
                            #   ignore a bigger selection that includes a folder
                            if len(filenames) == 1:
                                self.dlgAddImages.set_current_folder(filename)
                            valid = False
                            break

                # Remember this directory, and add the selected images
                if valid:
                    self.add_images_dir = self.dlgAddImages.get_current_folder()
                    self.add_images_to_gui(filenames)
                    done = True

        # Close the dialog
        self.dlgAddImages.hide()



    def copy_image(self, widget):
        """Copies the selected image."""

        sel = self.get_selection()

        # Check selection is valid
        if sel >= 0:
            # Insert copy after selected image
            report(2, _("Copying image at position {0}...").format(str(sel)), 0)
            report(2, self.gui_image_list[sel][IL_FILENAME], 1)

            sib = self.gui_image_list.get_iter_from_string(str(sel))
            src = self.gui_image_list[sel]
            self.gui_image_list.insert_after(sib, src)

            report(3, "Slideshow now has {0} image(s).".format(str(len(self.gui_image_list))), 1)
            report(3, "Success.", 0.5)

            # Select copy
            self.icvImages.select_path(sel + 1)

            # Update GUI
            self.on_select_image()
            self.check_if_saveable()


    def move_selected_image(self, relative_movement):
        """Moves the selected image by the specified amount."""

        current_position = self.get_selection()

        # Check selection is valid: image count; limits; neighbour-swap only
        if current_position >= 0:
            if relative_movement == -1:
                if self.is_selected_image_first():
                    return False
            elif relative_movement == 1:
                if self.is_selected_image_last():
                    return False
            else:
                return False

            # Swap image with appropriate neighbour
            a = self.gui_image_list.get_iter_from_string(str(current_position))
            b = self.gui_image_list.get_iter_from_string(str(current_position + relative_movement))
            self.gui_image_list.swap(a, b)

            # Update GUI
            self.on_select_image()


    def move_image_earlier(self, widget):
        """Move selected image before the previous image in the slideshow."""
        self.move_selected_image(-1)


    def move_image_later(self, widget):
        """Move selected image behind the next image in the slideshow."""
        self.move_selected_image(1)


    def remove_image(self, widget, sel=-1):
        """Removes the selected image from the slideshow."""

        # Get selection
        if sel == -1:
            sel = self.get_selection()

        # Check selection is valid
        if sel >= 0:
            # Remove the image
            going_pos = str(sel)
            going_path = self.gui_image_list[sel][IL_FILENAME]

            report(2, _("Removing image at position {0}...").format(going_pos))
            report(2, going_path, 1)

            self.gui_image_list.remove(self.gui_image_list.get_iter_from_string(going_pos))
            report(3, "Slideshow now has " + str(len(self.gui_image_list)) + " image(s).", 1)
            report(3, "Success.", 0.5)

            # Select neighbouring image
            ### If removed image wasn't last...
            if sel < len(self.gui_image_list):
                ### ...pick the following image
                self.icvImages.select_path(sel)

            ### Or, so long as there are still images remaining...
            elif len(self.gui_image_list) > 0:
                ### ...pick the previous image
                self.icvImages.select_path(sel - 1)

            # Update GUI
            self.check_if_saveable()
            self.check_if_clearable()


    def clear_images(self, widget):
        """Shows a confirmation dialogue."""

        # Check there are actually images to clear
        if len(self.gui_image_list) > 0:

            # Display confirmation dialog; focus Cancel button
            self.dlgConfirmClear.set_focus(self.btnKeep)
            response = self.dlgConfirmClear.run()
            self.dlgConfirmClear.hide()

            # Check for confirmation
            if response == 28:
                report(2, _("Clearing all images..."))
                self.gui_image_list.clear()

                self.check_if_saveable()
                self.check_if_clearable()
                report(3, "Slideshow now has {0} image(s).".format(str(len(self.gui_image_list))), 1)
                report(3, "Success.", 0.5)
            

    def focus_image_list(self):
        """Focuses the image list, so long as there are images in the slideshow."""

        if len(self.gui_image_list) > 0:
            self.icvImages.grab_focus()


    def icvImages_keyboard(self, widget, event):
        """Handles keyboard interaction with the IconView widget and image controls."""

        keyname = gtk.gdk.keyval_name(event.keyval)
        report(4, "{0} ({1}) key pressed while focused on the image list or controls".format(keyname, event.keyval))

        # Add & Copy
        if keyname == "Insert":
            if not (event.state and gtk.gdk.MOD1_MASK):
                # Add
                report(3, "Keyboard request to add image(s).")
                self.add_by_dialog(widget)
                self.focus_image_list()

            else:
                # Copy
                report(3, "Keyboard request to copy image.")
                self.copy_image(widget)
                self.focus_image_list()

        # Remove & Clear
        elif keyname == "Delete":
            if not (event.state and gtk.gdk.MOD1_MASK):
                # Remove
                report(3, "Keyboard request to remove image.")
                self.remove_image(widget)
                self.focus_image_list()

            else:
                # Clear
                report(3, "Keyboard request to clear images.")
                self.clear_images(widget)

        # Earlier and Later
        elif event.state and gtk.gdk.MOD1_MASK:
            if keyname == "Left":
                # Left
                report(3, "Keyboard request to move image earlier.")
                self.move_selected_image(-1)
                self.focus_image_list()

            elif keyname == "Right":
                # Right
                report(3, "Keyboard request to move image later.")
                self.move_selected_image(1)
                self.focus_image_list()


    def limit_length(self):
        """Places value and decimal-place limits on the slideshow length widgets."""

        #Maximum should be 92 days * dd_img_count + cumulative selecteds
        #Minimum should be 0.1seconds * dd_img_count + cumul. sels
        cumulative_selecteds = 0
        dd_img_count = 0
        for img in self.gui_image_list:
            if img[IL_USE_DEF_DURATION]:
                dd_img_count += 1
            else:
                cumulative_selecteds += img[IL_DUR_SCALED] * self.scales[img[IL_DUR_TIMESCALE]]

        s_max = 7948800 * dd_img_count + cumulative_selecteds
        s_min = 0.1 * dd_img_count + cumulative_selecteds

        scale = self.cmbSlideshowLength.get_active()
        adj   = self.SlideshowLength
        spin  = self.spnSlideshowLength

        if scale == 0:
            min_val = 1
            max_val = int(s_max / 86400)
            spin.set_digits(0)

        elif scale == 1:
            min_val = 1
            max_val = int(s_max / 3600)
            spin.set_digits(0)

        elif scale == 2:
            min_val = 1
            max_val = int(s_max / 60)
            spin.set_digits(0)

        elif scale == 3:
            min_val = s_min
            max_val = s_max
            spin.set_digits(1)

        # Set limits
        adj.set_lower(min_val)
        adj.set_upper(max_val)

        # Check value is within limits
        if  adj.get_value() < min_val:
            adj.set_value    (min_val)

        elif adj.get_value() > max_val:
            adj.set_value     (max_val)


    def limit_duration(self, which = 'default'):
        """Places value and decimal-place limits on selected duration spinbutton based on timescale."""

        # Select which spinbutton to place limits on
        if which == 'default':
            scale = self.cmbDurScaleDefault.get_active()
            adj   = self.DurDefault
            spin  = self.spnDurDefault
        else:
            scale = self.cmbDurScaleSelected.get_active()
            adj   = self.DurSelected
            spin  = self.spnDurSelected

        # Limit time to 92 days regardless of timescale
        if scale == 0:
            min_val = 1
            max_val = 2208
            spin.set_digits(0)

        elif scale == 1:
            min_val = 1
            max_val = 132480
            spin.set_digits(0)

        elif scale == 2:
            min_val = 0.1
            max_val = 7948800
            spin.set_digits(1)

        # Set limits
        adj.set_lower(min_val)
        adj.set_upper(max_val)

        # Check value is within limits
        if  adj.get_value() < min_val:
            adj.set_value    (min_val)

        elif adj.get_value() > max_val:
            adj.set_value     (max_val)


    def limit_default_transition(self):
        """Prevents the default transition from exceeding the default duration."""

        max_def_trans = min(self.default_duration, 86400)

        self.TrnDefault.set_upper(max_def_trans)

        if  self.TrnDefault.get_value() > max_def_trans:
            self.TrnDefault.set_value    (max_def_trans)


    def limit_selected_transition(self):
        """Prevents the selected transition from exceeding the selected duration."""

        max_sel_trans = min((self.DurSelected.get_value() * self.scales[self.cmbDurScaleSelected.get_active()]), 86400)

        self.TrnSelected.set_upper(max_sel_trans)

        if  self.TrnSelected.get_value() > max_sel_trans:
            self.TrnSelected.set_value    (max_sel_trans)


    def change_default_duration(self, widget, event=False):
        """Responds to changing the default duration."""

        self.default_duration_scaled = self.DurDefault.get_value()
        self.default_duration = self.default_duration_scaled * self.scales[self.default_scale]

        self.limit_default_transition()

        n = self.DurDefault.get_value()
        self.lstScalesDefault.set_value(self.lstScalesDefault.get_iter_from_string("0"), 0, N_("hour", "hours", n))
        self.lstScalesDefault.set_value(self.lstScalesDefault.get_iter_from_string("1"), 0, N_("minute", "minutes", n))
        self.lstScalesDefault.set_value(self.lstScalesDefault.get_iter_from_string("2"), 0, N_("second", "seconds", n))

        if self.chkUseDefDur.get_active():
            self.DurSelected.set_value(self.DurDefault.get_value())

        self.change_slideshow_length(widget)


    def change_selected_duration(self, widget, event=False):
        """Responds to changing the duration for the selected image."""

        self.limit_selected_transition()

        img_pos = self.get_selection()
        if img_pos > -1:
            self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), IL_DUR_SCALED, float(self.DurSelected.get_value()))

        n = self.DurSelected.get_value()
        self.lstScalesSelected.set_value(self.lstScalesSelected.get_iter_from_string("0"), 0, N_("hour", "hours", n))
        self.lstScalesSelected.set_value(self.lstScalesSelected.get_iter_from_string("1"), 0, N_("minute", "minutes", n))
        self.lstScalesSelected.set_value(self.lstScalesSelected.get_iter_from_string("2"), 0, N_("second", "seconds", n))

        self.change_slideshow_length(widget)


    def change_default_timescale(self, widget):
        """Responds to changing the default duration."""

        self.default_scale = self.cmbDurScaleDefault.get_active()
        self.default_duration = self.default_duration_scaled * self.scales[self.default_scale]

        self.limit_duration('default')
        self.limit_default_transition()

        if self.chkUseDefDur.get_active():
            self.cmbDurScaleSelected.set_active(self.cmbDurScaleDefault.get_active())
            self.limit_duration('selected')

        self.change_slideshow_length(widget)


    def change_selected_timescale(self, widget):
        """Responds to changing the timescale for the selected image."""

        self.limit_duration('selected')
        self.limit_selected_transition()

        img_pos = self.get_selection()
        if img_pos > -1:
            self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), IL_DUR_TIMESCALE, int(self.cmbDurScaleSelected.get_active()))

        self.change_slideshow_length(widget)


    def change_default_transition(self, widget, event=False):
        """Responds to changing the default transition."""

        self.default_transition = self.TrnDefault.get_value()

        if self.chkUseDefTrn.get_active():
            self.TrnSelected.set_value(self.TrnDefault.get_value())
            self.lblSecondsSelected.set_text(self.lblSecondsDefault.get_text())


    def change_selected_transition(self, widget, event=False):
        """Responds to changing the transition time for the selected image."""

        img_pos = self.get_selection()
        if img_pos > -1:
            self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), IL_TRANSITION, float(self.TrnSelected.get_value()))


    def toggle_time(self, widget):
        """Responds to toggling the 'Use default display time' checkbox."""

        img_pos = self.get_selection()

        if self.chkUseDefDur.get_active():
            if img_pos > -1:
                self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), IL_USE_DEF_DURATION, True)
                self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), IL_DUR_SCALED, float(self.DurDefault.get_value()))
                self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), IL_DUR_TIMESCALE, int(self.cmbDurScaleDefault.get_active()))
                self.spnDurSelected.set_value(self.gui_image_list[img_pos][IL_DUR_SCALED])
                self.cmbDurScaleSelected.set_active(self.gui_image_list[img_pos][IL_DUR_TIMESCALE])

            self.spnDurSelected.set_sensitive(False)
            self.cmbDurScaleSelected.set_sensitive(False)
        else:
            if img_pos > -1:
                self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), IL_USE_DEF_DURATION, False)

            self.spnDurSelected.set_sensitive(True)
            self.cmbDurScaleSelected.set_sensitive(True)

            self.change_slideshow_length(widget)


    def toggle_trans(self, widget):
        """Responds to toggling the 'Use default transition time' checkbox."""

        img_pos = self.get_selection()

        if self.chkUseDefTrn.get_active():
            if img_pos > -1:
                self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), IL_USE_DEF_TRANS, True)
                self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), IL_TRANSITION, float(self.TrnDefault.get_value()))
                self.spnTrnSelected.set_value(self.gui_image_list[img_pos][IL_TRANSITION])

            self.spnTrnSelected.set_sensitive(False)
            self.lblSecondsSelected.set_sensitive(False)
        else:
            if img_pos > -1:
                self.gui_image_list.set_value(self.gui_image_list.get_iter_from_string(str(img_pos)), IL_USE_DEF_TRANS, False)

            self.spnTrnSelected.set_sensitive(True)
            self.lblSecondsSelected.set_sensitive(True)


    def change_slideshow_length(self, widget=False, event=False):
        """Responds to a change in slideshow length."""

        # Check for singular/plural labels needed
        n = self.SlideshowLength.get_value()
        self.lstScalesSlideshow.set_value(self.lstScalesSlideshow.get_iter_from_string("0"), 0, N_("day", "days", n))
        self.lstScalesSlideshow.set_value(self.lstScalesSlideshow.get_iter_from_string("1"), 0, N_("hour", "hours", n))
        self.lstScalesSlideshow.set_value(self.lstScalesSlideshow.get_iter_from_string("2"), 0, N_("minute", "minutes", n))
        self.lstScalesSlideshow.set_value(self.lstScalesSlideshow.get_iter_from_string("3"), 0, N_("second", "seconds", n))

        # Try to fit to length
        if self.chkFitToLength.get_active():
            l_scale = (86400, 3600, 60, 1)
            s_length = float(self.SlideshowLength.get_value()) * float(l_scale[self.cmbSlideshowLength.get_active()])

            dd_img_count = 0
            for img in self.gui_image_list:
                if img[IL_USE_DEF_DURATION]:
                    dd_img_count += 1
                else:
                    s_length -= img[IL_DUR_SCALED] * self.scales[img[IL_DUR_TIMESCALE]]

            # Check for errors
            ### No images to adjust durations of
            if not dd_img_count:
                # Run error dialog
                self.dlgFitError.set_markup(_("<b>There are no longer any images using the default duration. Would you like to stop fitting the slideshow to length?</b>"))
                response = self.dlgFitError.run()
                self.dlgFitError.hide()

                # OK = Stop fitting to length
                if response == gtk.RESPONSE_OK:
                    self.chkFitToLength.set_active(False)
                    self.change_slideshow_length()

                # Cancel = undo changes
                else:
                    self.chkUseDefDur.set_active(True)

            ### Slideshow length is less than the minimum needed
            elif s_length < (0.1 * dd_img_count):
                self.chkFitToLength.set_active(False)

                # Run error dialog
                self.dlgFitError.set_markup(_("<b>The selected durations exceed the desired slideshow total. Would you like to stop fitting the slideshow to length?</b>"))
                response = self.dlgFitError.run()
                self.dlgFitError.hide()

                # OK = Stop fitting to length
                if response == gtk.RESPONSE_OK:
                    self.chkFitToLength.set_active(False)
                    self.change_slideshow_length()

                # Cancel = undo changes
                ### Selected duration was excessive: restore it
                elif widget == self.spnDurSelected or widget == self.cmbDurScaleSelected:
                    sel = self.DurSelected.get_value() * self.scales[self.cmbDurScaleSelected.get_active()]
                    sel += s_length - (dd_img_count * self.default_duration)

                    (duration, scale) = get_scaled_duration(sel)
                    self.DurSelected.set_value(duration)
                    self.cmbDurScaleSelected.set_active(scale)

                    self.chkFitToLength.set_active(True)

                ### Slideshow length was made too small: recalculate original
                else:
                    self.change_slideshow_length()
                    self.chkFitToLength.set_active(True)

            ### All conditions met: calculate default image duration
            else:
                d_duration = s_length / dd_img_count

                (duration, scale) = get_scaled_duration(d_duration)

                if duration < self.DurDefault.get_value():
                    self.cmbDurScaleDefault.set_active(scale)
                    self.DurDefault.set_value(duration)
                else:
                    self.DurDefault.set_value(duration)
                    self.cmbDurScaleDefault.set_active(scale)

        # Not trying to fit to length: just show the current total
        else:
            s_length = 0
            for img in self.gui_image_list:
                if img[IL_USE_DEF_DURATION]:
                    s_length += self.default_duration
                else:
                    s_length += img[IL_DUR_SCALED] * self.scales[img[IL_DUR_TIMESCALE]]

            (duration, scale) = get_scaled_duration(s_length, True)
            self.SlideshowLength.set_value(duration)
            self.cmbSlideshowLength.set_active(scale)

        # Update limits
        self.limit_length()


    def toggle_length(self, widget=False):
        """Responds to toggling the 'Fit to length' checkbox."""

        state = self.chkFitToLength.get_active()

        self.lblSlideshowLength.set_sensitive(state)
        self.spnSlideshowLength.set_sensitive(state)
        self.cmbSlideshowLength.set_sensitive(state)

        self.lblDurDefault.set_sensitive(not state)
        self.spnDurDefault.set_sensitive(not state)
        self.cmbDurScaleDefault.set_sensitive(not state)


    def name_changed(self, widget):
        """Responds to changing the name in the GUI."""

        self.check_if_saveable()


    def try_to_save(self, widget):
        """Tries to save slideshow after pressing ENTER in Name box."""

        if self.check_if_saveable():
            self.save_slideshow()


    def warn_file_overwrite(self):
        """Displays a file-overwrite warning dialog, and return True if user allows overwrite."""

        self.dlgOverwrite.set_markup('<b>' + _("\"{0}\" already exists. Do you want to replace it?" + '</b>').format(basename(self.output)))
        self.dlgOverwrite.set_focus(self.btnDontOverwrite)
        self.dlgOverwrite.show_all()

        response = self.dlgOverwrite.run()
        self.dlgOverwrite.hide()

        if response == 30:
            return True
        else:
            return False


    def warn_save_error(self, filename):
        """Displays an error dialog explaining that the slideshow couldn't be saved."""

        error_msg = _("An error was encountered while trying to save \"{0}\".").format(basename(filename)) + "\n\n"
        error_msg += _("Ensure that you have write access for the \"{0}\" directory.").format(dirname(filename))

        self.dlgSaveError.format_secondary_markup(error_msg)
        self.dlgSaveError.run()
        self.dlgSaveError.hide()


    def warn_set_error(self, s_name, registered = True, filename = ''):
        """Displays an error dialog explaining that the slideshow couldn't be set as the desktop background, then tries another method."""

        # Reset controls
        self.btnSetErrCancel.show()
        self.dlgSetError.set_focus(self.btnSetErrOK)

        # Construct error message
        if registered:
            error_msg = _("An error was encountered while trying to set the \"{0}\" slideshow as your desktop background.").format(s_name)
        else:
            error_msg = _("An error was encountered while trying to register the slideshow.")
            if filename:
                error_msg += ' ' + _("It has been saved as:") + "\n" + filename
            else:
                # There should always be a filename if it gets this far, but just in case...
                self.btnSetErrCancel.hide()

        if registered or filename:
            error_msg += "\n\n" + _("Would you like the Background preferences window to be opened so you can try to set it yourself?")

        self.dlgSetError.format_secondary_markup(error_msg)

        # Run the dialog
        response = self.dlgSetError.run()
        self.dlgSetError.hide()

        # Process response
        if response == gtk.RESPONSE_OK and (registered or filename):
            os.system("gnome-appearance-properties -p background {0} &".format(filename))


    def save_slideshow(self, widget=False):
        """Saves the slideshow."""

        # Convert GUI image list to basic image list
        image_list = []

        for image in self.gui_image_list:
            filename  = image[IL_FILENAME]

            if image[IL_USE_DEF_DURATION]:
                duration = self.default_duration
            else:
                duration  = image[IL_DUR_SCALED] * self.scales[image[IL_DUR_TIMESCALE]]

            if image[IL_USE_DEF_TRANS]:
                transition = self.default_transition
            else:
                transition = image[IL_TRANSITION]

            image_list.append((filename, duration, transition))

        # Create slideshow
        ### Name
        if self.use_gui_for_name:
            s_name = self.Name()
        else:
            s_name = self.name

        ### Filename
        if self.output:
            self.filename = basename(self.output)
        elif self.filename:
            self.output = join(self.directory, self.filename)
        elif s_name:
            self.filename = safe_filename(s_name)
            self.output = join(self.directory, self.filename)
        else:
            self.file_creation = False

        ### Creation
        s_xml = create_slideshow_xml(image_list, s_name)

        ### Echo XML
        if self.standard_output or self.verbosity >= 5:
            print s_xml

        # Save the slideshow
        if self.file_creation:

            # Check for overwrite
            if exists(self.output) and not self.force_overwrite:
                ok_to_write = self.warn_file_overwrite()
            else:
                ok_to_write = True

            # Write slideshow
            if ok_to_write:
                make_needed_dirs(self.output)

                slideshow_written = write_slideshow(self.output, s_xml)

                ### Error check
                if not slideshow_written:
                    self.warn_save_error(self.output)

                # Register wallpaper
                elif self.register_wallpaper:
                    #Get display style
                    sel = self.cmbSlideshowStyle.get_active()
                    wp_style = self.lstWallpaperStyles[sel][1]
                    
                    wallpaper_registered = register_wallpaper(s_name, self.output, wp_style)

                    # Set_wallpaper (not dependent on registration)
                    if self.set_wallpaper:
                        wallpaper_set = set_wallpaper(self.output, wp_style)

                        ### Error check
                        if not wallpaper_set:
                            self.warn_set_error(s_name, wallpaper_registered, self.output)

                        # Send notification
                        elif self.notify:
                            send_notification(s_name)

        # Close or Reset?
        if self.close_afterwards:
            gtk.main_quit()
        else:
            # Reset output and filename
            self.output = ''
            self.filename = ''

